@@ -12,10 +12,7 @@ GType gdata_service_error_get_type (void) G_GNUC_CONST;
 #define GDATA_TYPE_DATA_SERVICE_ERROR (gdata_service_error_get_type())
 GType gdata_authentication_error_get_type (void) G_GNUC_CONST;
 #define GDATA_TYPE_DATA_AUTHENTICATION_ERROR (gdata_authentication_error_get_type())
-/* enumerations from "gdata-media-rss.h" */
-GType gdata_media_expression_get_type (void) G_GNUC_CONST;
-#define GDATA_TYPE_DATA_MEDIA_EXPRESSION (gdata_media_expression_get_type())
-/* enumerations from "gdata-parser.h" */
+/* enumerations from "gdata-parsable.h" */
 GType gdata_parser_error_get_type (void) G_GNUC_CONST;
 #define GDATA_TYPE_DATA_PARSER_ERROR (gdata_parser_error_get_type())
 G_END_DECLS
