@@ -39,6 +39,11 @@
 #include "gdata-documents-query.h"
 #include "gdata-query.h"
 
+#include <gdata/services/documents/gdata-documents-spreadsheet.h>
+#include <gdata/services/documents/gdata-documents-presentation.h>
+#include <gdata/services/documents/gdata-documents-text.h>
+#include <gdata/services/documents/gdata-documents-folder.h>
+
 static void gdata_documents_query_finalize (GObject *object);
 static void gdata_documents_query_get_property (GObject *object, guint property_id, GValue *value, GParamSpec *pspec);
 static void gdata_documents_query_set_property (GObject *object, guint property_id, const GValue *value, GParamSpec *pspec);
@@ -223,27 +228,32 @@ static void
 get_query_uri (GDataQuery *self, const gchar *feed_uri, GString *query_uri, gboolean *params_started)
 {
 	GDataDocumentsQueryPrivate *priv = GDATA_DOCUMENTS_QUERY (self)->priv;
+	const gchar *entry_id = gdata_query_get_entry_id (self);
 
 	#define APPEND_SEP g_string_append_c (query_uri, (*params_started == FALSE) ? '?' : '&'); *params_started = TRUE;
 
-	if (priv->folder_id != NULL) {
-		g_string_append (query_uri, "/folder%%3A");
-		g_string_append_uri_escaped (query_uri, priv->folder_id, NULL, TRUE);
+	if (entry_id == NULL && priv->folder_id != NULL) {
+		g_string_append (query_uri, "/folder%3A");
+		g_string_append_uri_escaped (query_uri, priv->folder_id, NULL, FALSE);
 	}
 
 	/* Chain up to the parent class */
 	GDATA_QUERY_CLASS (gdata_documents_query_parent_class)->get_query_uri (self, feed_uri, query_uri, params_started);
 
+	/* Return if the entry ID has been set, since that's handled in the parent class' get_query_uri() function */
+	if (entry_id != NULL)
+		return;
+
 	if  (priv->collaborator_addresses != NULL) {
 		GList *collaborator_address;
 		APPEND_SEP
 		collaborator_address = priv->collaborator_addresses;
 
 		g_string_append (query_uri, "writer=");
-		g_string_append_uri_escaped (query_uri, gdata_gd_email_address_get_address (collaborator_address->data), NULL, TRUE);
+		g_string_append_uri_escaped (query_uri, gdata_gd_email_address_get_address (collaborator_address->data), NULL, FALSE);
 		for (collaborator_address = collaborator_address->next; collaborator_address != NULL; collaborator_address = collaborator_address->next) {
 			g_string_append_c (query_uri, ';');
-			g_string_append_uri_escaped (query_uri, gdata_gd_email_address_get_address (collaborator_address->data), NULL, TRUE);
+			g_string_append_uri_escaped (query_uri, gdata_gd_email_address_get_address (collaborator_address->data), NULL, FALSE);
 		}
 	}
 
@@ -253,17 +263,17 @@ get_query_uri (GDataQuery *self, const gchar *feed_uri, GString *query_uri, gboo
 		reader_address = priv->reader_addresses;
 
 		g_string_append (query_uri, "reader=");
-		g_string_append_uri_escaped (query_uri, gdata_gd_email_address_get_address (reader_address->data), NULL, TRUE);
+		g_string_append_uri_escaped (query_uri, gdata_gd_email_address_get_address (reader_address->data), NULL, FALSE);
 		for (reader_address = reader_address->next; reader_address != NULL; reader_address = reader_address->next) {
 			g_string_append_c (query_uri, ';');
-			g_string_append_uri_escaped (query_uri, gdata_gd_email_address_get_address (reader_address->data), NULL, TRUE);
+			g_string_append_uri_escaped (query_uri, gdata_gd_email_address_get_address (reader_address->data), NULL, FALSE);
 		}
 	}
 
 	if (priv->title != NULL) {
 		APPEND_SEP
 		g_string_append (query_uri, "title=");
-		g_string_append_uri_escaped (query_uri, priv->title, NULL, TRUE);
+		g_string_append_uri_escaped (query_uri, priv->title, NULL, FALSE);
 		if (priv->exact_title == TRUE)
 			g_string_append (query_uri, "&title-exact=true");
 	}
