@@ -12,9 +12,13 @@ GType gdata_youtube_standard_feed_type_get_type (void) G_GNUC_CONST;
 #define GDATA_TYPE_YOUTUBE_STANDARD_FEED_TYPE (gdata_youtube_standard_feed_type_get_type())
 GType gdata_youtube_service_error_get_type (void) G_GNUC_CONST;
 #define GDATA_TYPE_YOUTUBE_SERVICE_ERROR (gdata_youtube_service_error_get_type())
-/* enumerations from "gdata-youtube-query.h" */
+/* enumerations from "gdata-youtube-video.h" */
+GType gdata_youtube_aspect_ratio_get_type (void) G_GNUC_CONST;
+#define GDATA_TYPE_YOUTUBE_ASPECT_RATIO (gdata_youtube_aspect_ratio_get_type())
+/* enumerations from "gdata-youtube-content.h" */
 GType gdata_youtube_format_get_type (void) G_GNUC_CONST;
 #define GDATA_TYPE_YOUTUBE_FORMAT (gdata_youtube_format_get_type())
+/* enumerations from "gdata-youtube-query.h" */
 GType gdata_youtube_safe_search_get_type (void) G_GNUC_CONST;
 #define GDATA_TYPE_YOUTUBE_SAFE_SEARCH (gdata_youtube_safe_search_get_type())
 GType gdata_youtube_sort_order_get_type (void) G_GNUC_CONST;
