@@ -62,6 +62,23 @@ test_remove_all_documents_and_folders (GDataService *service)
 	g_assert (service != NULL);
 
 	query = gdata_documents_query_new (NULL);
+	gdata_documents_query_set_show_folders (query, FALSE);
+
+	feed = gdata_documents_service_query_documents (GDATA_DOCUMENTS_SERVICE (service), query, NULL, NULL, NULL, &error);
+	g_assert_no_error (error);
+	g_assert (GDATA_IS_DOCUMENTS_FEED (feed));
+
+	/* We delete the folders after all the files so we don't get ETag mismatches; deleting a folder changes the version
+	 * of all the documents inside it. Conversely, deleting an entry inside a folder changes the version of the folder. */
+	for (i = gdata_feed_get_entries (GDATA_FEED (feed)); i != NULL; i = i->next) {
+		gdata_service_delete_entry (GDATA_SERVICE (service), GDATA_ENTRY (i->data), NULL, &error);
+		g_assert_no_error (error);
+		g_clear_error (&error);
+	}
+
+	g_object_unref (feed);
+
+	/* Now delete the folders */
 	gdata_documents_query_set_show_folders (query, TRUE);
 
 	feed = gdata_documents_service_query_documents (GDATA_DOCUMENTS_SERVICE (service), query, NULL, NULL, NULL, &error);
@@ -71,11 +88,9 @@ test_remove_all_documents_and_folders (GDataService *service)
 	for (i = gdata_feed_get_entries (GDATA_FEED (feed)); i != NULL; i = i->next) {
 		gdata_service_delete_entry (GDATA_SERVICE (service), GDATA_ENTRY (i->data), NULL, &error);
 		g_assert_no_error (error);
+		g_clear_error (&error);
 	}
 
-	g_clear_error (&error);
-
-	/* TODO: check entries and feed properties */
 	g_object_unref (feed);
 }
 
@@ -201,9 +216,9 @@ test_upload_metadata_file (GDataService *service)
 }
 
 static void
-test_upload_file (GDataService *service)
+test_upload_file_get_entry (GDataService *service)
 {
-	GDataDocumentsEntry *new_document;
+	GDataDocumentsEntry *new_document, *newly_created_presentation;
 	GFile *document_file;
 	GDataCategory *category;
 	GError *error = NULL;
@@ -211,7 +226,6 @@ test_upload_file (GDataService *service)
 	g_assert (service != NULL);
 
 	document_file = g_file_new_for_path (TEST_FILE_DIR "test.ppt");
-
 	category = gdata_category_new ("http://schemas.google.com/docs/2007#presentation", "http://schemas.google.com/g/2005#kind", "presentation");
 
 	/* Insert the document */
@@ -219,8 +233,15 @@ test_upload_file (GDataService *service)
 	g_assert_no_error (error);
 	g_assert (GDATA_IS_DOCUMENTS_PRESENTATION (new_document));
 
+	/* Get the entry on the server */
+	newly_created_presentation = gdata_documents_service_query_single_document (GDATA_DOCUMENTS_SERVICE (service), GDATA_TYPE_DOCUMENTS_PRESENTATION,
+										    gdata_documents_entry_get_document_id (new_document), NULL, &error);
+	g_assert_no_error (error);
+	g_assert (GDATA_IS_DOCUMENTS_PRESENTATION (newly_created_presentation));
+
 	g_clear_error (&error);
 	g_object_unref (new_document);
+	g_object_unref (newly_created_presentation);
 	g_object_unref (document_file);
 }
 
@@ -464,37 +485,67 @@ test_download_all_documents (GDataService *service)
 	GDataDocumentsFeed *feed;
 	GError *error = NULL;
 	gchar *content_type = NULL;
-	GFile *destination_directory;
+	GFile *destination_file;
+	gchar *destination_file_name;
+	GString *destination_display_name;
 	GList *i;
+	gint ods_nb = 0, odt_nb = 0;
 
-	destination_directory = g_file_new_for_path ("/tmp");
 	feed = gdata_documents_service_query_documents (GDATA_DOCUMENTS_SERVICE (service), NULL, NULL, NULL, NULL, &error);
 	g_assert_no_error (error);
 	g_assert (GDATA_IS_FEED (feed));
 
 	for (i = gdata_feed_get_entries (GDATA_FEED (feed)); i != NULL; i = i->next) {
-		GFile *destination_file = NULL;
-
 		if (GDATA_IS_DOCUMENTS_PRESENTATION (i->data)) {
+			destination_file = g_file_new_for_path ("/tmp");
 			destination_file = gdata_documents_presentation_download_document (GDATA_DOCUMENTS_PRESENTATION (i->data), GDATA_DOCUMENTS_SERVICE (service),
-											   &content_type, GDATA_DOCUMENTS_PRESENTATION_PPT, destination_directory,
+											   &content_type, GDATA_DOCUMENTS_PRESENTATION_PPT, destination_file,
 											   TRUE, NULL, &error);
+
 		} else if (GDATA_IS_DOCUMENTS_SPREADSHEET (i->data)) {
+			destination_file_name = g_strdup_printf ("/tmp/%s.%s", gdata_documents_entry_get_document_id (GDATA_DOCUMENTS_ENTRY (i->data)), "ods");
+			destination_file = g_file_new_for_path (destination_file_name);
+			g_free (destination_file_name);
 			destination_file = gdata_documents_spreadsheet_download_document (i->data, GDATA_DOCUMENTS_SERVICE (service),
-											  &content_type, GDATA_DOCUMENTS_SPREADSHEET_ODS, -1, destination_directory,
+											  &content_type, GDATA_DOCUMENTS_SPREADSHEET_ODS, -1, destination_file,
 											  TRUE, NULL, &error);
+			g_assert_no_error (error);
+
+			destination_display_name = g_string_new (gdata_entry_get_title (GDATA_ENTRY(i->data)));
+			g_string_append (destination_display_name, ".ods");
+			g_file_set_display_name (destination_file, destination_display_name->str, NULL, &error);
+			while (g_error_matches (error, G_IO_ERROR, G_IO_ERROR_EXISTS)) {
+				g_clear_error (&error);
+				error = NULL;
+				g_string_printf (destination_display_name,"%s%d.%s", gdata_entry_get_title (GDATA_ENTRY(i->data)), ods_nb, "ods");
+				g_file_set_display_name (destination_file, destination_display_name->str, NULL, &error);
+				ods_nb++;
+			}
+			g_string_free (destination_display_name, TRUE);
 		} else if (GDATA_IS_DOCUMENTS_TEXT (i->data)) {
+			destination_file_name = g_strdup_printf ("/tmp/%s.%s", gdata_documents_entry_get_document_id (GDATA_DOCUMENTS_ENTRY (i->data)), "odt");
+			destination_file = g_file_new_for_path (destination_file_name);
+			g_free (destination_file_name);
 			destination_file = gdata_documents_text_download_document (i->data, GDATA_DOCUMENTS_SERVICE (service), &content_type, GDATA_DOCUMENTS_TEXT_ODT,
-										   destination_directory, TRUE, NULL, &error);
+										   destination_file, TRUE, NULL, &error);
+			g_assert_no_error (error);
+
+			destination_display_name = g_string_new (gdata_entry_get_title (GDATA_ENTRY(i->data)));
+			g_string_append (destination_display_name, ".odt");
+			g_file_set_display_name (destination_file, destination_display_name->str, NULL, &error);
+			while (g_error_matches (error, G_IO_ERROR, G_IO_ERROR_EXISTS)) {
+				g_clear_error (&error);
+				g_string_printf (destination_display_name,"%s%d.%s", gdata_entry_get_title (GDATA_ENTRY(i->data)), odt_nb, "odt");
+				g_file_set_display_name (destination_file, destination_display_name->str, NULL, &error);
+				odt_nb++;
+			}
+			g_string_free (destination_display_name, TRUE);
 		}
 
 		g_assert_no_error (error);
-		if (destination_file != NULL)
-			g_object_unref (destination_file);
 		g_free (content_type);
 	}
 
-	g_object_unref (destination_directory);
 	g_object_unref (feed);
 	g_clear_error (&error);
 }
@@ -556,13 +607,11 @@ main (int argc, char *argv[])
 
 	g_test_add_data_func ("/documents/remove/all", service, test_remove_all_documents_and_folders);
 
-	g_test_add_data_func ("/documents/upload/only_file", service, test_upload_file);
+	g_test_add_data_func ("/documents/upload/only_file_get_entry", service, test_upload_file_get_entry);
 	g_test_add_data_func ("/documents/upload/metadata_file", service, test_upload_metadata_file);
 	g_test_add_data_func ("/documents/upload/only_metadata", service, test_upload_metadata);
 	g_test_add_data_func ("/documents/upload/metadata_file_in_new_folder", service, test_upload_file_metadata_in_new_folder);
-
 	g_test_add_data_func ("/documents/download/download_all_documents", service, test_download_all_documents);
-
 	g_test_add_data_func ("/documents/update/only_metadata", service, test_update_metadata);
 	g_test_add_data_func ("/documents/update/only_file", service, test_update_file);
 	g_test_add_data_func ("/documents/update/metadata_file", service, test_update_metadata_file);
@@ -580,6 +629,7 @@ main (int argc, char *argv[])
 	g_test_add_data_func ("/documents/move/remove_from_folder", service, test_add_remove_file_from_folder);
 
 	g_test_add_data_func ("/documents/remove/all", service, test_remove_all_documents_and_folders);
+
 	retval = g_test_run ();
 
 	g_object_unref (service);
