@@ -206,34 +206,59 @@ test_query_categories (void)
 	/* AND */
 	gdata_query_set_categories (query, "Fritz/Laurie");
 	query_uri = gdata_query_get_query_uri (query, "http://example.com");
-
 	g_assert_cmpstr (query_uri, ==, "http://example.com/-/Fritz/Laurie?q=foobar");
 	g_free (query_uri);
 
 	/* OR */
 	gdata_query_set_categories (query, "Fritz|Laurie");
 	query_uri = gdata_query_get_query_uri (query, "http://example.com");
-
 	g_assert_cmpstr (query_uri, ==, "http://example.com/-/Fritz%7CLaurie?q=foobar");
 	g_free (query_uri);
 
 	/* Combination */
 	gdata_query_set_categories (query, "A|-{urn:google.com}B/-C");
 	query_uri = gdata_query_get_query_uri (query, "http://example.com/gdata_test");
-
 	g_assert_cmpstr (query_uri, ==, "http://example.com/gdata_test/-/A%7C-%7Burn%3Agoogle.com%7DB/-C?q=foobar");
 	g_free (query_uri);
 
 	/* Same combination without q param */
 	gdata_query_set_q (query, NULL);
 	query_uri = gdata_query_get_query_uri (query, "http://example.com");
-
 	g_assert_cmpstr (query_uri, ==, "http://example.com/-/A%7C-%7Burn%3Agoogle.com%7DB/-C");
 	g_free (query_uri);
 
 	g_object_unref (query);
 }
 
+static void
+test_query_unicode (void)
+{
+	GDataQuery *query;
+	gchar *query_uri;
+
+	g_test_bug ("602497");
+
+	/* Simple query */
+	query = gdata_query_new ("fööbar‽");
+	query_uri = gdata_query_get_query_uri (query, "http://example.com");
+	g_assert_cmpstr (query_uri, ==, "http://example.com?q=f%C3%B6%C3%B6bar%E2%80%BD");
+	g_free (query_uri);
+
+	/* Categories */
+	gdata_query_set_categories (query, "Ümlauts|¿Questions‽");
+	query_uri = gdata_query_get_query_uri (query, "http://example.com");
+	g_assert_cmpstr (query_uri, ==, "http://example.com/-/%C3%9Cmlauts%7C%C2%BFQuestions%E2%80%BD?q=f%C3%B6%C3%B6bar%E2%80%BD");
+	g_free (query_uri);
+
+	/* Author */
+	gdata_query_set_author (query, "Lørd Brïan Bleßêd");
+	query_uri = gdata_query_get_query_uri (query, "http://example.com");
+	g_assert_cmpstr (query_uri, ==, "http://example.com/-/%C3%9Cmlauts%7C%C2%BFQuestions%E2%80%BD?q=f%C3%B6%C3%B6bar%E2%80%BD&author=L%C3%B8rd%20Br%C3%AFan%20Ble%C3%9F%C3%AAd");
+	g_free (query_uri);
+
+	g_object_unref (query);
+}
+
 static void
 test_color_parsing (void)
 {
@@ -390,7 +415,7 @@ test_atom_category (void)
 	xml = gdata_parsable_get_xml (GDATA_PARSABLE (category));
 	g_assert_cmpstr (xml, ==,
 			 "<category xmlns='http://www.w3.org/2005/Atom' "
-			 	"term='jokes' scheme='http://foobar.com#categories' label='Jokes &amp; Trivia'/>");
+				"term='jokes' scheme='http://foobar.com#categories' label='Jokes &amp; Trivia'/>");
 	g_free (xml);
 	g_object_unref (category);
 
@@ -421,8 +446,8 @@ test_atom_category (void)
 	xml = gdata_parsable_get_xml (GDATA_PARSABLE (category));
 	g_assert_cmpstr (xml, ==,
 			 "<category xmlns='http://www.w3.org/2005/Atom' term='documentary'>"
-			 	"<foobar/>"
-			 	"<shizzle/>"
+				"<foobar/>"
+				"<shizzle/>"
 			 "</category>");
 	g_free (xml);
 	g_object_unref (category);
@@ -500,7 +525,7 @@ test_atom_link (void)
 	xml = gdata_parsable_get_xml (GDATA_PARSABLE (link));
 	g_assert_cmpstr (xml, ==,
 			 "<link xmlns='http://www.w3.org/2005/Atom' href='http://example.com/' title='All About Angle Brackets: &lt;, &gt;' "
-			 	"rel='http://test.com#link-type' type='text/plain' hreflang='de' length='2000'/>");
+				"rel='http://test.com#link-type' type='text/plain' hreflang='de' length='2000'/>");
 	g_free (xml);
 	g_object_unref (link);
 
@@ -523,7 +548,7 @@ test_atom_link (void)
 	xml = gdata_parsable_get_xml (GDATA_PARSABLE (link));
 	g_assert_cmpstr (xml, ==,
 			 "<link xmlns='http://www.w3.org/2005/Atom' href='http://shizzle.com' rel='http://www.iana.org/assignments/relation/alternate'>"
-			 	"Test Content<foobar/></link>");
+				"Test Content<foobar/></link>");
 	g_free (xml);
 	g_object_unref (link);
 }
@@ -561,7 +586,7 @@ test_gd_email_address (void)
 	xml = gdata_parsable_get_xml (GDATA_PARSABLE (email));
 	g_assert_cmpstr (xml, ==,
 			 "<gd:email xmlns='http://www.w3.org/2005/Atom' xmlns:gd='http://schemas.google.com/g/2005' address='fubar@gmail.com' "
-			 	"rel='http://schemas.google.com/g/2005#home' label='Personal &amp; Private' primary='true'/>");
+				"rel='http://schemas.google.com/g/2005#home' label='Personal &amp; Private' primary='true'/>");
 	g_free (xml);
 	g_object_unref (email);
 
@@ -582,7 +607,7 @@ test_gd_email_address (void)
 	xml = gdata_parsable_get_xml (GDATA_PARSABLE (email));
 	g_assert_cmpstr (xml, ==,
 			 "<gd:email xmlns='http://www.w3.org/2005/Atom' xmlns:gd='http://schemas.google.com/g/2005' address='test@example.com' "
-			 	"primary='false'/>");
+				"primary='false'/>");
 	g_free (xml);
 	g_object_unref (email);
 }
@@ -621,7 +646,7 @@ test_gd_im_address (void)
 	xml = gdata_parsable_get_xml (GDATA_PARSABLE (im));
 	g_assert_cmpstr (xml, ==,
 			 "<gd:im xmlns='http://www.w3.org/2005/Atom' xmlns:gd='http://schemas.google.com/g/2005' "
-			 	"address='foo@bar.msn.com' protocol='http://schemas.google.com/g/2005#MSN' "
+				"address='foo@bar.msn.com' protocol='http://schemas.google.com/g/2005#MSN' "
 				"rel='http://schemas.google.com/g/2005#home' primary='true'/>");
 	g_free (xml);
 	g_object_unref (im);
@@ -644,11 +669,90 @@ test_gd_im_address (void)
 	xml = gdata_parsable_get_xml (GDATA_PARSABLE (im));
 	g_assert_cmpstr (xml, ==,
 			 "<gd:im xmlns='http://www.w3.org/2005/Atom' xmlns:gd='http://schemas.google.com/g/2005' address='foo@baz.example.com' "
-			 	"label='Other &amp; Miscellaneous' primary='false'/>");
+				"label='Other &amp; Miscellaneous' primary='false'/>");
 	g_free (xml);
 	g_object_unref (im);
 }
 
+static void
+test_gd_name (void)
+{
+	GDataGDName *name, *name2;
+	gchar *xml;
+	GError *error = NULL;
+
+	name = GDATA_GD_NAME (gdata_parsable_new_from_xml (GDATA_TYPE_GD_NAME,
+		"<gd:name xmlns:gd='http://schemas.google.com/g/2005'>"
+			"<gd:givenName>Brian</gd:givenName>"
+			"<gd:additionalName>Charles</gd:additionalName>"
+			"<gd:familyName>Blessed</gd:familyName>"
+			"<gd:namePrefix>Mr</gd:namePrefix>"
+			"<gd:nameSuffix>ABC</gd:nameSuffix>"
+			"<gd:fullName>Mr Brian Charles Blessed, ABC</gd:fullName>"
+		"</gd:name>", -1, &error));
+	g_assert_no_error (error);
+	g_assert (GDATA_IS_GD_NAME (name));
+	g_clear_error (&error);
+
+	/* Check the properties */
+	g_assert_cmpstr (gdata_gd_name_get_given_name (name), ==, "Brian");
+	g_assert_cmpstr (gdata_gd_name_get_additional_name (name), ==, "Charles");
+	g_assert_cmpstr (gdata_gd_name_get_family_name (name), ==, "Blessed");
+	g_assert_cmpstr (gdata_gd_name_get_prefix (name), ==, "Mr");
+	g_assert_cmpstr (gdata_gd_name_get_suffix (name), ==, "ABC");
+	g_assert_cmpstr (gdata_gd_name_get_full_name (name), ==, "Mr Brian Charles Blessed, ABC");
+
+	/* Compare it against another identical name */
+	name2 = gdata_gd_name_new ("Brian", "Blessed");
+	gdata_gd_name_set_additional_name (name2, "Charles");
+	gdata_gd_name_set_prefix (name2, "Mr");
+	gdata_gd_name_set_suffix (name2, "ABC");
+	g_assert_cmpint (gdata_gd_name_compare (name, name2), ==, 0);
+
+	/* …and a different one */
+	gdata_gd_name_set_prefix (name2, "Mrs");
+	g_assert_cmpint (gdata_gd_name_compare (name, name2), !=, 0);
+	g_object_unref (name2);
+
+	/* Check the outputted XML is the same */
+	xml = gdata_parsable_get_xml (GDATA_PARSABLE (name));
+	g_assert_cmpstr (xml, ==,
+			 "<gd:name xmlns='http://www.w3.org/2005/Atom' xmlns:gd='http://schemas.google.com/g/2005'>"
+				"<gd:givenName>Brian</gd:givenName>"
+				"<gd:additionalName>Charles</gd:additionalName>"
+				"<gd:familyName>Blessed</gd:familyName>"
+				"<gd:namePrefix>Mr</gd:namePrefix>"
+				"<gd:nameSuffix>ABC</gd:nameSuffix>"
+				"<gd:fullName>Mr Brian Charles Blessed, ABC</gd:fullName>"
+			 "</gd:name>");
+	g_free (xml);
+	g_object_unref (name);
+
+	/* Now parse an address with less information available */
+	name = GDATA_GD_NAME (gdata_parsable_new_from_xml (GDATA_TYPE_GD_NAME,
+		"<gd:name xmlns:gd='http://schemas.google.com/g/2005'><gd:givenName>Bob</gd:givenName></gd:name>", -1, &error));
+	g_assert_no_error (error);
+	g_assert (GDATA_IS_GD_NAME (name));
+	g_clear_error (&error);
+
+	/* Check the properties */
+	g_assert_cmpstr (gdata_gd_name_get_given_name (name), ==, "Bob");
+	g_assert (gdata_gd_name_get_additional_name (name) == NULL);
+	g_assert (gdata_gd_name_get_family_name (name) == NULL);
+	g_assert (gdata_gd_name_get_prefix (name) == NULL);
+	g_assert (gdata_gd_name_get_suffix (name) == NULL);
+	g_assert (gdata_gd_name_get_full_name (name) == NULL);
+
+	/* Check the outputted XML is still correct */
+	xml = gdata_parsable_get_xml (GDATA_PARSABLE (name));
+	g_assert_cmpstr (xml, ==,
+			 "<gd:name xmlns='http://www.w3.org/2005/Atom' xmlns:gd='http://schemas.google.com/g/2005'>"
+				"<gd:givenName>Bob</gd:givenName>"
+			 "</gd:name>");
+	g_free (xml);
+	g_object_unref (name);
+}
+
 static void
 test_gd_organization (void)
 {
@@ -661,6 +765,9 @@ test_gd_organization (void)
 			"primary='true'>"
 			"<gd:orgName>Google, Inc.</gd:orgName>"
 			"<gd:orgTitle>&lt;Angle Bracketeer&gt;</gd:orgTitle>"
+			"<gd:orgDepartment>Finance</gd:orgDepartment>"
+			"<gd:orgJobDescription>Doing stuff.</gd:orgJobDescription>"
+			"<gd:orgSymbol>FOO</gd:orgSymbol>"
 		"</gd:organization>", -1, &error));
 	g_assert_no_error (error);
 	g_assert (GDATA_IS_GD_ORGANIZATION (org));
@@ -671,10 +778,14 @@ test_gd_organization (void)
 	g_assert_cmpstr (gdata_gd_organization_get_title (org), ==, "<Angle Bracketeer>");
 	g_assert_cmpstr (gdata_gd_organization_get_relation_type (org), ==, "http://schemas.google.com/g/2005#work");
 	g_assert_cmpstr (gdata_gd_organization_get_label (org), ==, "Work & Occupation");
+	g_assert_cmpstr (gdata_gd_organization_get_department (org), ==, "Finance");
+	g_assert_cmpstr (gdata_gd_organization_get_job_description (org), ==, "Doing stuff.");
+	g_assert_cmpstr (gdata_gd_organization_get_symbol (org), ==, "FOO");
 	g_assert (gdata_gd_organization_is_primary (org) == TRUE);
 
 	/* Compare it against another identical organization */
 	org2 = gdata_gd_organization_new ("Google, Inc.", "<Angle Bracketeer>", "http://schemas.google.com/g/2005#work", "Work & Occupation", TRUE);
+	gdata_gd_organization_set_department (org2, "Finance");
 	g_assert_cmpint (gdata_gd_organization_compare (org, org2), ==, 0);
 
 	/* …and a different one */
@@ -686,9 +797,12 @@ test_gd_organization (void)
 	xml = gdata_parsable_get_xml (GDATA_PARSABLE (org));
 	g_assert_cmpstr (xml, ==,
 			 "<gd:organization xmlns='http://www.w3.org/2005/Atom' xmlns:gd='http://schemas.google.com/g/2005' "
-			 	"rel='http://schemas.google.com/g/2005#work' label='Work &amp; Occupation' primary='true'>"
+				"rel='http://schemas.google.com/g/2005#work' label='Work &amp; Occupation' primary='true'>"
 				"<gd:orgName>Google, Inc.</gd:orgName>"
 				"<gd:orgTitle>&lt;Angle Bracketeer&gt;</gd:orgTitle>"
+				"<gd:orgDepartment>Finance</gd:orgDepartment>"
+				"<gd:orgJobDescription>Doing stuff.</gd:orgJobDescription>"
+				"<gd:orgSymbol>FOO</gd:orgSymbol>"
 			 "</gd:organization>");
 	g_free (xml);
 	g_object_unref (org);
@@ -706,6 +820,9 @@ test_gd_organization (void)
 	g_assert (gdata_gd_organization_get_relation_type (org) == NULL);
 	g_assert (gdata_gd_organization_get_label (org) == NULL);
 	g_assert (gdata_gd_organization_is_primary (org) == FALSE);
+	g_assert (gdata_gd_organization_get_department (org) == NULL);
+	g_assert (gdata_gd_organization_get_job_description (org) == NULL);
+	g_assert (gdata_gd_organization_get_symbol (org) == NULL);
 
 	/* Check the outputted XML contains the unknown XML */
 	xml = gdata_parsable_get_xml (GDATA_PARSABLE (org));
@@ -750,8 +867,8 @@ test_gd_phone_number (void)
 	xml = gdata_parsable_get_xml (GDATA_PARSABLE (phone));
 	g_assert_cmpstr (xml, ==,
 			 "<gd:phoneNumber xmlns='http://www.w3.org/2005/Atom' xmlns:gd='http://schemas.google.com/g/2005' "
-			 	"uri='tel:+12065551212' rel='http://schemas.google.com/g/2005#mobile' label='Personal &amp; business calls only' "
-			 	"primary='false'>+1 206 555 1212</gd:phoneNumber>");
+				"uri='tel:+12065551212' rel='http://schemas.google.com/g/2005#mobile' label='Personal &amp; business calls only' "
+				"primary='false'>+1 206 555 1212</gd:phoneNumber>");
 	g_free (xml);
 	g_object_unref (phone);
 
@@ -773,7 +890,7 @@ test_gd_phone_number (void)
 	xml = gdata_parsable_get_xml (GDATA_PARSABLE (phone));
 	g_assert_cmpstr (xml, ==,
 			 "<gd:phoneNumber xmlns='http://www.w3.org/2005/Atom' "
-			 	"xmlns:gd='http://schemas.google.com/g/2005' primary='false'>(425) 555-8080 ext. 72585</gd:phoneNumber>");
+				"xmlns:gd='http://schemas.google.com/g/2005' primary='false'>(425) 555-8080 ext. 72585</gd:phoneNumber>");
 	g_free (xml);
 	g_object_unref (phone);
 }
@@ -786,49 +903,58 @@ test_gd_postal_address (void)
 	GError *error = NULL;
 
 	postal = GDATA_GD_POSTAL_ADDRESS (gdata_parsable_new_from_xml (GDATA_TYPE_GD_POSTAL_ADDRESS,
-		"<gd:postalAddress xmlns:gd='http://schemas.google.com/g/2005' label='Home &amp; Safe House' "
+		"<gd:structuredPostalAddress xmlns:gd='http://schemas.google.com/g/2005' label='Home &amp; Safe House' "
 			"rel='http://schemas.google.com/g/2005#home' primary='true'>"
-			" \t 500 West 45th Street\nNew York, NY 10036  \n  "
-		"</gd:postalAddress>", -1, &error));
+			"<gd:street>500 West 45th Street</gd:street>"
+			"<gd:city>New York</gd:city>"
+			"<gd:postcode>NY 10036</gd:postcode>"
+		"</gd:structuredPostalAddress>", -1, &error));
 	g_assert_no_error (error);
 	g_assert (GDATA_IS_GD_POSTAL_ADDRESS (postal));
 	g_clear_error (&error);
 
 	/* Check the properties */
-	g_assert_cmpstr (gdata_gd_postal_address_get_address (postal), ==, "500 West 45th Street\nNew York, NY 10036");
+	g_assert_cmpstr (gdata_gd_postal_address_get_street (postal), ==, "500 West 45th Street");
+	g_assert_cmpstr (gdata_gd_postal_address_get_city (postal), ==, "New York");
+	g_assert_cmpstr (gdata_gd_postal_address_get_postcode (postal), ==, "NY 10036");
 	g_assert_cmpstr (gdata_gd_postal_address_get_relation_type (postal), ==, "http://schemas.google.com/g/2005#home");
 	g_assert_cmpstr (gdata_gd_postal_address_get_label (postal), ==, "Home & Safe House");
 	g_assert (gdata_gd_postal_address_is_primary (postal) == TRUE);
 
 	/* Compare it against another identical address */
-	postal2 = gdata_gd_postal_address_new ("500 West 45th Street\nNew York, NY 10036", "http://schemas.google.com/g/2005#home",
-					       "Home & Safe House", TRUE);
+	postal2 = gdata_gd_postal_address_new ("http://schemas.google.com/g/2005#home", "Home & Safe House", TRUE);
+	gdata_gd_postal_address_set_street (postal2, "500 West 45th Street");
+	gdata_gd_postal_address_set_city (postal2, "New York");
+	gdata_gd_postal_address_set_postcode (postal2, "NY 10036");
 	g_assert_cmpint (gdata_gd_postal_address_compare (postal, postal2), ==, 0);
 
 	/* …and a different one */
-	gdata_gd_postal_address_set_address (postal2, "Atlas Mountains");
+	gdata_gd_postal_address_set_city (postal2, "Atlas Mountains");
 	g_assert_cmpint (gdata_gd_postal_address_compare (postal, postal2), !=, 0);
 	g_object_unref (postal2);
 
 	/* Check the outputted XML is the same */
 	xml = gdata_parsable_get_xml (GDATA_PARSABLE (postal));
 	g_assert_cmpstr (xml, ==,
-			 "<gd:postalAddress xmlns='http://www.w3.org/2005/Atom' xmlns:gd='http://schemas.google.com/g/2005' "
-			 	"rel='http://schemas.google.com/g/2005#home' label='Home &amp; Safe House' primary='true'>"
-				"500 West 45th Street\nNew York, NY 10036"
-			 "</gd:postalAddress>");
+			 "<gd:structuredPostalAddress xmlns='http://www.w3.org/2005/Atom' xmlns:gd='http://schemas.google.com/g/2005' "
+				"rel='http://schemas.google.com/g/2005#home' label='Home &amp; Safe House' primary='true'>"
+				"<gd:street>500 West 45th Street</gd:street>"
+				"<gd:city>New York</gd:city>"
+				"<gd:postcode>NY 10036</gd:postcode>"
+			 "</gd:structuredPostalAddress>");
 	g_free (xml);
 	g_object_unref (postal);
 
 	/* Now parse an address with less information available */
 	postal = GDATA_GD_POSTAL_ADDRESS (gdata_parsable_new_from_xml (GDATA_TYPE_GD_POSTAL_ADDRESS,
-		"<gd:postalAddress xmlns:gd='http://schemas.google.com/g/2005'>f</gd:postalAddress>", -1, &error));
+		"<gd:structuredPostalAddress xmlns:gd='http://schemas.google.com/g/2005'><gd:street>f</gd:street></gd:structuredPostalAddress>",
+		-1, &error));
 	g_assert_no_error (error);
 	g_assert (GDATA_IS_GD_POSTAL_ADDRESS (postal));
 	g_clear_error (&error);
 
 	/* Check the properties */
-	g_assert_cmpstr (gdata_gd_postal_address_get_address (postal), ==, "f");
+	g_assert_cmpstr (gdata_gd_postal_address_get_street (postal), ==, "f");
 	g_assert (gdata_gd_postal_address_get_relation_type (postal) == NULL);
 	g_assert (gdata_gd_postal_address_get_label (postal) == NULL);
 	g_assert (gdata_gd_postal_address_is_primary (postal) == FALSE);
@@ -836,8 +962,8 @@ test_gd_postal_address (void)
 	/* Check the outputted XML contains the unknown XML */
 	xml = gdata_parsable_get_xml (GDATA_PARSABLE (postal));
 	g_assert_cmpstr (xml, ==,
-			 "<gd:postalAddress xmlns='http://www.w3.org/2005/Atom' xmlns:gd='http://schemas.google.com/g/2005' primary='false'>"
-				 "f</gd:postalAddress>");
+			 "<gd:structuredPostalAddress xmlns='http://www.w3.org/2005/Atom' xmlns:gd='http://schemas.google.com/g/2005' primary='false'>"
+				 "<gd:street>f</gd:street></gd:structuredPostalAddress>");
 	g_free (xml);
 	g_object_unref (postal);
 }
@@ -922,7 +1048,7 @@ test_gd_reminder (void)
 	xml = gdata_parsable_get_xml (GDATA_PARSABLE (reminder));
 	g_assert_cmpstr (xml, ==,
 			 "<gd:reminder xmlns='http://www.w3.org/2005/Atom' xmlns:gd='http://schemas.google.com/g/2005' "
-			 	"absoluteTime='2005-06-07T00:55:00Z' method='alert'/>");
+				"absoluteTime='2005-06-07T00:55:00Z' method='alert'/>");
 	g_free (xml);
 	g_object_unref (reminder);
 }
@@ -968,7 +1094,7 @@ test_gd_when (void)
 	xml = gdata_parsable_get_xml (GDATA_PARSABLE (when));
 	g_assert_cmpstr (xml, ==,
 			 "<gd:when xmlns='http://www.w3.org/2005/Atom' xmlns:gd='http://schemas.google.com/g/2005' startTime='2005-06-07T01:00:00Z' "
-			 	"endTime='2005-06-07T02:00:00Z'/>");
+				"endTime='2005-06-07T02:00:00Z'/>");
 	g_free (xml);
 	g_object_unref (when);
 
@@ -1003,7 +1129,7 @@ test_gd_when (void)
 	xml = gdata_parsable_get_xml (GDATA_PARSABLE (when));
 	g_assert_cmpstr (xml, ==,
 			 "<gd:when xmlns='http://www.w3.org/2005/Atom' xmlns:gd='http://schemas.google.com/g/2005' startTime='2005-06-06' "
-			 	"endTime='2005-06-08' valueString='This weekend'>"
+				"endTime='2005-06-08' valueString='This weekend'>"
 				"<gd:reminder minutes='15'/>"
 				"<foobar/>"
 			 "</gd:when>");
@@ -1043,8 +1169,8 @@ test_gd_where (void)
 	xml = gdata_parsable_get_xml (GDATA_PARSABLE (where));
 	g_assert_cmpstr (xml, ==,
 			 "<gd:where xmlns='http://www.w3.org/2005/Atom' xmlns:gd='http://schemas.google.com/g/2005' "
-			 	"label='New York Location &lt;videoconference&gt;' rel='http://schemas.google.com/g/2005#event.alternate' "
-			 	"valueString='Metropolis'/>");
+				"label='New York Location &lt;videoconference&gt;' rel='http://schemas.google.com/g/2005#event.alternate' "
+				"valueString='Metropolis'/>");
 	g_free (xml);
 	g_object_unref (where);
 
@@ -1064,7 +1190,7 @@ test_gd_where (void)
 	xml = gdata_parsable_get_xml (GDATA_PARSABLE (where));
 	g_assert_cmpstr (xml, ==,
 			 "<gd:where xmlns='http://www.w3.org/2005/Atom' xmlns:gd='http://schemas.google.com/g/2005' "
-			 	"valueString='Google Cafeteria &lt;Building 40&gt;'/>");
+				"valueString='Google Cafeteria &lt;Building 40&gt;'/>");
 	g_free (xml);
 	g_object_unref (where);
 }
@@ -1101,7 +1227,7 @@ test_gd_who (void)
 	xml = gdata_parsable_get_xml (GDATA_PARSABLE (who));
 	g_assert_cmpstr (xml, ==,
 			 "<gd:who xmlns='http://www.w3.org/2005/Atom' xmlns:gd='http://schemas.google.com/g/2005' email='liz@example.com' "
-			 	"rel='http://schemas.google.com/g/2005#message.to' valueString='Elizabeth'/>");
+				"rel='http://schemas.google.com/g/2005#message.to' valueString='Elizabeth'/>");
 	g_free (xml);
 	g_object_unref (who);
 
@@ -1125,6 +1251,307 @@ test_gd_who (void)
 	g_object_unref (who);
 }
 
+static void
+test_media_category (void)
+{
+	GDataMediaCategory *category;
+	gchar *xml;
+	GError *error = NULL;
+
+	category = GDATA_MEDIA_CATEGORY (gdata_parsable_new_from_xml (GDATA_TYPE_MEDIA_CATEGORY,
+		"<media:category xmlns:media='http://search.yahoo.com/mrss/' scheme='http://dmoz.org' "
+			"label='Ace Ventura - Pet &amp; Detective'>Arts/Movies/Titles/A/Ace_Ventura_Series/Ace_Ventura_-_Pet_Detective"
+			"</media:category>", -1, &error));
+	g_assert_no_error (error);
+	g_assert (GDATA_IS_MEDIA_CATEGORY (category));
+	g_clear_error (&error);
+
+	/* Check the properties */
+	g_assert_cmpstr (gdata_media_category_get_category (category), ==, "Arts/Movies/Titles/A/Ace_Ventura_Series/Ace_Ventura_-_Pet_Detective");
+	g_assert_cmpstr (gdata_media_category_get_scheme (category), ==, "http://dmoz.org");
+	g_assert_cmpstr (gdata_media_category_get_label (category), ==, "Ace Ventura - Pet & Detective");
+
+	/* Check the outputted XML is the same */
+	xml = gdata_parsable_get_xml (GDATA_PARSABLE (category));
+	g_assert_cmpstr (xml, ==,
+			 "<media:category xmlns='http://www.w3.org/2005/Atom' xmlns:media='http://search.yahoo.com/mrss/' "
+				"scheme='http://dmoz.org' "
+				"label='Ace Ventura - Pet &amp; Detective'>Arts/Movies/Titles/A/Ace_Ventura_Series/Ace_Ventura_-_Pet_Detective"
+				"</media:category>");
+	g_free (xml);
+	g_object_unref (category);
+
+	/* Now parse one with less information available */
+	category = GDATA_MEDIA_CATEGORY (gdata_parsable_new_from_xml (GDATA_TYPE_MEDIA_CATEGORY,
+		"<media:category xmlns:media='http://search.yahoo.com/mrss/'>foo</media:category>", -1, &error));
+	g_assert_no_error (error);
+	g_assert (GDATA_IS_MEDIA_CATEGORY (category));
+	g_clear_error (&error);
+
+	/* Check the properties */
+	g_assert_cmpstr (gdata_media_category_get_category (category), ==, "foo");
+	g_assert_cmpstr (gdata_media_category_get_scheme (category), ==, "http://video.search.yahoo.com/mrss/category_schema");
+	g_assert (gdata_media_category_get_label (category) == NULL);
+
+	/* Check the outputted XML is the same */
+	xml = gdata_parsable_get_xml (GDATA_PARSABLE (category));
+	g_assert_cmpstr (xml, ==,
+			 "<media:category xmlns='http://www.w3.org/2005/Atom' xmlns:media='http://search.yahoo.com/mrss/' "
+				"scheme='http://video.search.yahoo.com/mrss/category_schema'>foo</media:category>");
+	g_free (xml);
+	g_object_unref (category);
+}
+
+static void
+test_media_content (void)
+{
+	GDataMediaContent *content;
+	GError *error = NULL;
+
+	content = GDATA_MEDIA_CONTENT (gdata_parsable_new_from_xml (GDATA_TYPE_MEDIA_CONTENT,
+		"<media:content xmlns:media='http://search.yahoo.com/mrss/' url='http://www.foo.com/movie.mov' fileSize='12216320' "
+			"type='video/quicktime' medium='video' isDefault='true' expression='nonstop' duration='185' height='200' width='300'/>",
+			-1, &error));
+	g_assert_no_error (error);
+	g_assert (GDATA_IS_MEDIA_CONTENT (content));
+	g_clear_error (&error);
+
+	/* Check the properties */
+	g_assert_cmpstr (gdata_media_content_get_uri (content), ==, "http://www.foo.com/movie.mov");
+	g_assert_cmpint (gdata_media_content_get_filesize (content), ==, 12216320);
+	g_assert_cmpstr (gdata_media_content_get_content_type (content), ==, "video/quicktime");
+	g_assert (gdata_media_content_get_medium (content) == GDATA_MEDIA_VIDEO);
+	g_assert (gdata_media_content_is_default (content) == TRUE);
+	g_assert (gdata_media_content_get_expression (content) == GDATA_MEDIA_EXPRESSION_NONSTOP);
+	g_assert_cmpint (gdata_media_content_get_duration (content), ==, 185);
+	g_assert_cmpuint (gdata_media_content_get_width (content), ==, 300);
+	g_assert_cmpuint (gdata_media_content_get_height (content), ==, 200);
+
+	/* NOTE: We don't check the outputted XML, since the class currently doesn't have any support for outputting XML */
+	g_object_unref (content);
+
+	/* Now parse one with less information available */
+	content = GDATA_MEDIA_CONTENT (gdata_parsable_new_from_xml (GDATA_TYPE_MEDIA_CONTENT,
+		"<media:content xmlns:media='http://search.yahoo.com/mrss/' url='http://foobar.com/'/>", -1, &error));
+	g_assert_no_error (error);
+	g_assert (GDATA_IS_MEDIA_CONTENT (content));
+	g_clear_error (&error);
+
+	/* Check the properties */
+	g_assert_cmpstr (gdata_media_content_get_uri (content), ==, "http://foobar.com/");
+	g_assert_cmpint (gdata_media_content_get_filesize (content), ==, 0);
+	g_assert (gdata_media_content_get_content_type (content) == NULL);
+	g_assert (gdata_media_content_get_medium (content) == GDATA_MEDIA_UNKNOWN);
+	g_assert (gdata_media_content_is_default (content) == FALSE);
+	g_assert (gdata_media_content_get_expression (content) == GDATA_MEDIA_EXPRESSION_FULL);
+	g_assert_cmpint (gdata_media_content_get_duration (content), ==, 0);
+	g_assert_cmpuint (gdata_media_content_get_width (content), ==, 0);
+	g_assert_cmpuint (gdata_media_content_get_height (content), ==, 0);
+
+	g_object_unref (content);
+}
+
+static void
+test_media_credit (void)
+{
+	GDataMediaCredit *credit;
+	GError *error = NULL;
+
+	credit = GDATA_MEDIA_CREDIT (gdata_parsable_new_from_xml (GDATA_TYPE_MEDIA_CREDIT,
+		"<media:credit xmlns:media='http://search.yahoo.com/mrss/' role='producer' scheme='urn:foobar'>entity name</media:credit>",
+			-1, &error));
+	g_assert_no_error (error);
+	g_assert (GDATA_IS_MEDIA_CREDIT (credit));
+	g_clear_error (&error);
+
+	/* Check the properties */
+	g_assert_cmpstr (gdata_media_credit_get_credit (credit), ==, "entity name");
+	g_assert_cmpstr (gdata_media_credit_get_scheme (credit), ==, "urn:foobar");
+	g_assert_cmpstr (gdata_media_credit_get_role (credit), ==, "producer");
+
+	/* NOTE: We don't check the outputted XML, since the class currently doesn't have any support for outputting XML */
+	g_object_unref (credit);
+
+	/* Now parse one with less information available */
+	credit = GDATA_MEDIA_CREDIT (gdata_parsable_new_from_xml (GDATA_TYPE_MEDIA_CREDIT,
+		"<media:credit xmlns:media='http://search.yahoo.com/mrss/'>John Smith</media:credit>", -1, &error));
+	g_assert_no_error (error);
+	g_assert (GDATA_IS_MEDIA_CREDIT (credit));
+	g_clear_error (&error);
+
+	/* Check the properties */
+	g_assert_cmpstr (gdata_media_credit_get_credit (credit), ==, "John Smith");
+	g_assert_cmpstr (gdata_media_credit_get_scheme (credit), ==, "urn:ebu");
+	g_assert (gdata_media_credit_get_role (credit) == NULL);
+
+	g_object_unref (credit);
+}
+
+#if 0
+/* We can't test GDataMediaGroup, since it isn't currently publicly exposed */
+static void
+test_media_group (void)
+{
+	GDataMediaGroup *group;
+	GList *contents, *thumbnails;
+	gchar *xml;
+	GError *error = NULL;
+
+	group = GDATA_MEDIA_GROUP (gdata_parsable_new_from_xml (GDATA_TYPE_MEDIA_GROUP,
+		"<media:group xmlns:media='http://search.yahoo.com/mrss/'>"
+			"<media:title>Foobar — shizzle!</media:title>"
+			"<media:description>This is a description, isn't it‽</media:description>"
+			"<media:keywords>keywords,are, fun</media:keywords>"
+			"<media:category scheme='http://dmoz.org' label='Ace Ventura - Pet Detective'>"
+				"Arts/Movies/Titles/A/Ace_Ventura_Series/Ace_Ventura_-_Pet_Detective</media:category>"
+			"<media:content url='http://foobar.com/'/>"
+			"<media:content url='http://www.foo.com/movie.mov' fileSize='12216320' type='video/quicktime' medium='video' isDefault='true' "
+				"expression='nonstop' duration='185' height='200' width='300'/>"
+			"<media:credit>John Smith</media:credit>"
+			"<media:player url='http://www.foo.com/player?id=1111' height='200' width='400'/>"
+			"<media:restriction relationship='deny'>all</media:restriction>"
+			"<media:restriction relationship='allow' type='country'>au us</media:restriction>"
+			"<media:thumbnail url='http://www.foo.com/keyframe.jpg' width='75' height='50' time='12:05:01.123'/>"
+			"<media:thumbnail url='http://www.foo.com/keyframe0.jpg' time='00:00:00'/>"
+		"</media:group>", -1, &error));
+	g_assert_no_error (error);
+	g_assert (GDATA_IS_MEDIA_GROUP (group));
+	g_clear_error (&error);
+
+	/* Check the properties */
+	g_assert_cmpstr (gdata_media_group_get_title (group), ==, "Foobar — shizzle!");
+	g_assert_cmpstr (gdata_media_group_get_description (group), ==, "This is a description, isn't it‽");
+	g_assert_cmpstr (gdata_media_group_get_keywords (group), ==, "keywords,are, fun");
+	g_assert (GDATA_IS_MEDIA_CATEGORY (gdata_media_group_get_category (group)));
+	g_assert (GDATA_IS_MEDIA_CONTENT (gdata_media_group_look_up_content (group, NULL)));
+	g_assert (GDATA_IS_MEDIA_CONTENT (gdata_media_group_look_up_content (group, "video/quicktime")));
+
+	contents = gdata_media_group_get_contents (group);
+	g_assert_cmpuint (g_list_length (contents), ==, 2);
+	g_assert (GDATA_IS_MEDIA_CONTENT (contents->data));
+	g_assert (GDATA_IS_MEDIA_CONTENT (contents->next->data));
+
+	g_assert (GDATA_IS_MEDIA_CREDIT (gdata_media_group_get_credit (group)));
+	g_assert_cmpstr (gdata_media_group_get_player_uri (group), ==, "http://www.foo.com/player?id=1111");
+	g_assert (gdata_media_group_is_restricted_in_country (group, "uk") == TRUE);
+	g_assert (gdata_media_group_is_restricted_in_country (group, "au") == FALSE);
+	g_assert (gdata_media_group_is_restricted_in_country (group, "us") == FALSE);
+
+	thumbnails = gdata_media_group_get_thumbnails (group);
+	g_assert_cmpuint (g_list_length (thumbnails), ==, 2);
+	g_assert (GDATA_IS_MEDIA_THUMBNAIL (thumbnails->data));
+	g_assert (GDATA_IS_MEDIA_THUMBNAIL (thumbnails->next->data));
+
+	/* Check the outputted XML is the same */
+	xml = gdata_parsable_get_xml (GDATA_PARSABLE (group));
+	g_assert_cmpstr (xml, ==,
+			 "<media:group xmlns:media='http://search.yahoo.com/mrss/'>"
+				"<media:title>Foobar — shizzle!</media:title>"
+				"<media:description>This is a description, isn't it‽</media:description>"
+				"<media:keywords>keywords,are, fun</media:keywords>"
+				"<media:category scheme='http://dmoz.org' label='Ace Ventura - Pet Detective'>"
+					"Arts/Movies/Titles/A/Ace_Ventura_Series/Ace_Ventura_-_Pet_Detective</media:category>"
+				"<media:content url='http://foobar.com/'/>"
+				"<media:content url='http://www.foo.com/movie.mov' fileSize='12216320' type='video/quicktime' medium='video' "
+					"isDefault='true' expression='nonstop' duration='185' height='200' width='300'/>"
+				"<media:credit>John Smith</media:credit>"
+				"<media:player url='http://www.foo.com/player?id=1111' height='200' width='400'/>"
+				"<media:restriction relationship='deny'>all</media:restriction>"
+				"<media:restriction relationship='allow' type='country'>au us</media:restriction>"
+				"<media:thumbnail url='http://www.foo.com/keyframe.jpg' width='75' height='50' time='12:05:01.123'/>"
+				"<media:thumbnail url='http://www.foo.com/keyframe0.jpg' time='00:00:00'/>"
+			 "</media:group>");
+	g_free (xml);
+
+	/* Check setting things works */
+	gdata_media_group_set_title (group, "Test title");
+	g_assert_cmpstr (gdata_media_group_get_title (group), ==, "Test title");
+	gdata_media_group_set_title (group, NULL);
+	g_assert (gdata_media_group_get_title (group) == NULL);
+
+	gdata_media_group_set_description (group, "Foobar");
+	g_assert_cmpstr (gdata_media_group_get_description (group), ==, "Foobar");
+	gdata_media_group_set_description (group, NULL);
+	g_assert (gdata_media_group_get_description (group) == NULL);
+
+	gdata_media_group_set_keywords (group, "a,b, c");
+	g_assert_cmpstr (gdata_media_group_get_keywords (group), ==, "a,b, c");
+	gdata_media_group_set_keywords (group, NULL);
+	g_assert (gdata_media_group_get_keywords (group) == NULL);
+
+	gdata_media_group_set_category (group, NULL);
+	g_assert (gdata_media_group_get_category (group) == NULL);
+
+	g_object_unref (group);
+
+	/* Now parse one with less information available */
+	group = GDATA_MEDIA_GROUP (gdata_parsable_new_from_xml (GDATA_TYPE_MEDIA_GROUP,
+		"<media:group xmlns:media='http://search.yahoo.com/mrss/'></media:group>", -1, &error));
+	g_assert_no_error (error);
+	g_assert (GDATA_IS_MEDIA_GROUP (group));
+	g_clear_error (&error);
+
+	/* Check the properties */
+	g_assert (gdata_media_group_get_title (group) == NULL);
+	g_assert (gdata_media_group_get_description (group) == NULL);
+	g_assert (gdata_media_group_get_keywords (group) == NULL);
+	g_assert (gdata_media_group_get_category (group) == NULL);
+	g_assert (gdata_media_group_look_up_content (group, NULL) == NULL);
+	g_assert (gdata_media_group_look_up_content (group, "video/quicktime") == NULL);
+	g_assert (gdata_media_group_get_contents (group) == NULL);
+	g_assert (gdata_media_group_get_credit (group) == NULL);
+	g_assert (gdata_media_group_get_player_uri (group) == NULL);
+	g_assert (gdata_media_group_is_restricted_in_country (group, "uk") == FALSE);
+	g_assert (gdata_media_group_is_restricted_in_country (group, "au") == FALSE);
+	g_assert (gdata_media_group_is_restricted_in_country (group, "us") == FALSE);
+	g_assert (gdata_media_group_get_thumbnails (group) == NULL);
+
+	/* Check the outputted XML is the same */
+	xml = gdata_parsable_get_xml (GDATA_PARSABLE (group));
+	g_assert_cmpstr (xml, ==, "<media:group xmlns='http://www.w3.org/2005/Atom' xmlns:media='http://search.yahoo.com/mrss/'></media:group>");
+	g_free (xml);
+	g_object_unref (group);
+}
+#endif
+
+static void
+test_media_thumbnail (void)
+{
+	GDataMediaThumbnail *thumbnail;
+	GError *error = NULL;
+
+	thumbnail = GDATA_MEDIA_THUMBNAIL (gdata_parsable_new_from_xml (GDATA_TYPE_MEDIA_THUMBNAIL,
+		"<media:thumbnail xmlns:media='http://search.yahoo.com/mrss/' url='http://www.foo.com/keyframe.jpg' width='75' height='50' "
+			"time='12:05:01.123'/>", -1, &error));
+	g_assert_no_error (error);
+	g_assert (GDATA_IS_MEDIA_THUMBNAIL (thumbnail));
+	g_clear_error (&error);
+
+	/* Check the properties */
+	g_assert_cmpstr (gdata_media_thumbnail_get_uri (thumbnail), ==, "http://www.foo.com/keyframe.jpg");
+	g_assert_cmpuint (gdata_media_thumbnail_get_width (thumbnail), ==, 75);
+	g_assert_cmpuint (gdata_media_thumbnail_get_height (thumbnail), ==, 50);
+	g_assert_cmpint (gdata_media_thumbnail_get_time (thumbnail), ==, 43501123);
+
+	/* NOTE: We don't check the outputted XML, since the class currently doesn't have any support for outputting XML */
+	g_object_unref (thumbnail);
+
+	/* Now parse one with less information available */
+	thumbnail = GDATA_MEDIA_THUMBNAIL (gdata_parsable_new_from_xml (GDATA_TYPE_MEDIA_THUMBNAIL,
+		"<media:thumbnail xmlns:media='http://search.yahoo.com/mrss/' url='http://foobar.com/'/>", -1, &error));
+	g_assert_no_error (error);
+	g_assert (GDATA_IS_MEDIA_THUMBNAIL (thumbnail));
+	g_clear_error (&error);
+
+	/* Check the properties */
+	g_assert_cmpstr (gdata_media_thumbnail_get_uri (thumbnail), ==, "http://foobar.com/");
+	g_assert_cmpuint (gdata_media_thumbnail_get_width (thumbnail), ==, 0);
+	g_assert_cmpuint (gdata_media_thumbnail_get_height (thumbnail), ==, 0);
+	g_assert_cmpint (gdata_media_thumbnail_get_time (thumbnail), ==, -1);
+
+	g_object_unref (thumbnail);
+}
+
 int
 main (int argc, char *argv[])
 {
@@ -1135,6 +1562,7 @@ main (int argc, char *argv[])
 	g_test_add_func ("/entry/get_xml", test_entry_get_xml);
 	g_test_add_func ("/entry/parse_xml", test_entry_parse_xml);
 	g_test_add_func ("/query/categories", test_query_categories);
+	g_test_add_func ("/query/unicode", test_query_unicode);
 	g_test_add_func ("/color/parsing", test_color_parsing);
 	g_test_add_func ("/color/output", test_color_output);
 
@@ -1145,6 +1573,7 @@ main (int argc, char *argv[])
 
 	g_test_add_func ("/gd/email_address", test_gd_email_address);
 	g_test_add_func ("/gd/im_address", test_gd_im_address);
+	g_test_add_func ("/gd/name", test_gd_name);
 	g_test_add_func ("/gd/organization", test_gd_organization);
 	g_test_add_func ("/gd/phone_number", test_gd_phone_number);
 	g_test_add_func ("/gd/postal_address", test_gd_postal_address);
@@ -1153,6 +1582,11 @@ main (int argc, char *argv[])
 	g_test_add_func ("/gd/where", test_gd_where);
 	g_test_add_func ("/gd/who", test_gd_who);
 
+	g_test_add_func ("/media/category", test_media_category);
+	g_test_add_func ("/media/content", test_media_content);
+	g_test_add_func ("/media/credit", test_media_credit);
+	/* g_test_add_func ("/media/group", test_media_group); */
+	g_test_add_func ("/media/thumbnail", test_media_thumbnail);
 	/*g_test_add_data_func ("/media/thumbnail/parse_time", "", test_media_thumbnail_parse_time);
 	g_test_add_data_func ("/media/thumbnail/parse_time", "de_DE", test_media_thumbnail_parse_time);*/
 
